@@ -21,6 +21,22 @@
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#ifndef THREADING_SCHEME
+#define THREADING_SCHEME 6
+#endif
+
+#ifdef GFORTH_DEBUGGING
+/* schedule the ip update after the rest of the primitive;
+   never mind speed */
+#undef THREADING_SCHEME
+#ifdef DIRECT_THREADED
+#define THREADING_SCHEME 10
+#else
+#define THREADING_SCHEME 8
+#endif /* DIRECT_THREADED */
+#endif /* GFORTH_DEBUGGING */
+
+
 /* define SYSCALL */
 
 #ifndef SYSCALL
@@ -83,5 +99,6 @@ typedef void *Label;
 #define HAS_HASH
 #define HAS_OS
 #define HAS_XCONDS
+#define HAS_STANDARDTHREADING
 
 #define RELINFOBITS	8
