@@ -72,10 +72,14 @@ int offset_image=0;
 int die_on_signal=0;
 #ifndef INCLUDE_IMAGE
 static int clear_dictionary=0;
-static size_t pagesize=0;
+UCell pagesize=1;
+char *progname;
+#else
+char *progname = "gforth";
+int optind = 1;
 #endif
 static int debug=0;
-char *progname;
+ImageHeader *gforth_header;
 
 /* image file format:
  *  "#! binary-path -i\n" (e.g., "#! /usr/local/bin/gforth-0.4.0 -i\n")
@@ -89,7 +93,7 @@ char *progname;
  *              bit 0:   endian, big=0, little=1.
  *  The magic are always 8 octets, no matter what the native AU/character size is
  *  padding to max alignment (no padding necessary on current machines)
- *  ImageHeader structure (see below)
+ *  ImageHeader structure (see forth.h)
  *  data (size in ImageHeader.image_size)
  *  tags ((if relocatable, 1 bit/data cell)
  *
@@ -104,28 +108,6 @@ char *progname;
  * If the word is <CF(DOESJUMP), it's a primitive
  */
 
-typedef struct {
-  Address base;		/* base address of image (0 if relocatable) */
-  UCell checksum;	/* checksum of ca's to protect against some
-			   incompatible	binary/executable combinations
-			   (0 if relocatable) */
-  UCell image_size;	/* all sizes in bytes */
-  UCell dict_size;
-  UCell data_stack_size;
-  UCell fp_stack_size;
-  UCell return_stack_size;
-  UCell locals_stack_size;
-  Xt *boot_entry;	/* initial ip for booting (in BOOT) */
-  Xt *throw_entry;	/* ip after signal (in THROW) */
-  Cell unused1;		/* possibly tib stack size */
-  Cell unused2;
-  Address data_stack_base; /* this and the following fields are initialized by the loader */
-  Address fp_stack_base;
-  Address return_stack_base;
-  Address locals_stack_base;
-} ImageHeader;
-/* the image-header is created in main.fs */
-
 void relocate(Cell *image, const char *bitstring, int size, Label symbols[])
 {
   int i=0, j, k, steps=(size/sizeof(Cell))/RELINFOBITS;
@@ -165,6 +147,7 @@ void relocate(Cell *image, const char *bitstring, int size, Label symbols[])
 	  image[i]+=(Cell)image;
       }
     }
+  ((ImageHeader*)(image))->base = (Address) image;
 }
 
 UCell checksum(Label symbols[])
@@ -216,8 +199,8 @@ Address my_alloc(Cell size)
     fprintf(stderr,"try mmap($%lx, $%lx, ..., MAP_ANON, ...); ", (long)next_address, (long)size);
   r=mmap(next_address, size, PROT_EXEC|PROT_READ|PROT_WRITE, MAP_ANON|MAP_PRIVATE, -1, 0);
 #else /* !defined(MAP_ANON) */
-  /* Ultrix (at least does not define MAP_FILE and MAP_PRIVATE (both are
-     apparently defaults*/
+  /* Ultrix (at least) does not define MAP_FILE and MAP_PRIVATE (both are
+     apparently defaults) */
 #ifndef MAP_FILE
 # define MAP_FILE 0
 #endif
@@ -243,7 +226,7 @@ Address my_alloc(Cell size)
   if (r != (Address)-1) {
     if (debug)
       fprintf(stderr, "success, address=$%lx\n", (long) r);
-    if (pagesize != 0)
+    if (pagesize != 1)
       next_address = (Address)(((((Cell)r)+size-1)&-pagesize)+2*pagesize); /* leave one page unmapped */
     return r;
   }
@@ -297,25 +280,24 @@ void alloc_stacks(ImageHeader * header)
 
 int go_forth(Address image, int stack, Cell *entries)
 {
-  Cell *sp=(Cell*)(((ImageHeader *)image)->data_stack_base + dsize);
-  Float *fp=(Float *)(((ImageHeader *)image)->fp_stack_base + fsize);
-  Cell *rp=(Cell *)(((ImageHeader *)image)->return_stack_base + rsize);
-  Address lp=((ImageHeader *)image)->locals_stack_base + lsize;
-  Xt *ip=(Xt *)(((ImageHeader *)image)->boot_entry);
+  ImageHeader *image_header = (ImageHeader *)image;
+  Cell *sp0=(Cell*)(image_header->data_stack_base + dsize);
+  Float *fp0=(Float *)(image_header->fp_stack_base + fsize);
+  Cell *rp0=(Cell *)(image_header->return_stack_base + rsize);
+  Address lp0=image_header->locals_stack_base + lsize;
+  Xt *ip0=(Xt *)(image_header->boot_entry);
 #ifdef SYSSIGNALS
   int throw_code;
 #endif
 
   /* ensure that the cached elements (if any) are accessible */
-  IF_TOS(sp--);
-  IF_FTOS(fp--);
+  IF_TOS(sp0--);
+  IF_FTOS(fp0--);
   
   for(;stack>0;stack--)
-    *--sp=entries[stack-1];
+    *--sp0=entries[stack-1];
 
-#if !defined(MSDOS) && !defined(SHARC) && !defined(_WIN32) && !defined(__EMX__)
   get_winsize();
-#endif
    
 #ifdef SYSSIGNALS
   install_signal_handlers(); /* right place? */
@@ -326,13 +308,37 @@ int go_forth(Address image, int stack, Cell *entries)
     static Float signal_fp_stack[1];
 
     signal_data_stack[7]=throw_code;
+
+#ifdef GFORTH_DEBUGGING
+    if (rp <= rp0 && rp > (Cell *)(image_header->return_stack_base+5)) {
+      /* no rstack overflow or underflow */
+      rp0 = rp;
+      *--rp0 = (Cell)ip;
+    }
+    else /* I love non-syntactic ifdefs :-) */
+#endif
+    rp0 = signal_return_stack+8;
+    /* fprintf(stderr, "rp=$%x\n",rp0);*/
     
-    return((int)engine(((ImageHeader *)image)->throw_entry,signal_data_stack+7,
-		       signal_return_stack+8,signal_fp_stack,0));
+    return((int)engine(image_header->throw_entry, signal_data_stack+7,
+		       rp0, signal_fp_stack, 0));
   }
 #endif
 
-  return((int)engine(ip,sp,rp,fp,lp));
+  return((int)engine(ip0,sp0,rp0,fp0,lp0));
+}
+
+
+void print_sizes(Cell sizebyte)
+     /* print size information */
+{
+  static char* endianstring[]= { "   big","little" };
+  
+  fprintf(stderr,"%s endian, cell=%d bytes, char=%d bytes, au=%d bytes\n",
+	  endianstring[sizebyte & 1],
+	  1 << ((sizebyte >> 1) & 3),
+	  1 << ((sizebyte >> 3) & 3),
+	  1 << ((sizebyte >> 5) & 3));
 }
 
 #ifndef INCLUDE_IMAGE
@@ -342,12 +348,12 @@ Address loader(FILE *imagefile, char* filename)
   ImageHeader header;
   Address image;
   Address imp; /* image+preamble */
-  Char magic[9];
+  Char magic[8];
+  char magic7; /* size byte of magic number */
   Cell preamblesize=0;
   Label *symbols = engine(0,0,0,0,0);
   Cell data_offset = offset_image ? 56*sizeof(Cell) : 0;
   UCell check_sum;
-  static char* endianstring[]= { "big","little" };
   Cell ausize = ((RELINFOBITS ==  8) ? 0 :
 		 (RELINFOBITS == 16) ? 1 :
 		 (RELINFOBITS == 32) ? 2 : 3);
@@ -357,6 +363,13 @@ Address loader(FILE *imagefile, char* filename)
   Cell cellsize = ((sizeof(Cell) == 1) ? 0 :
 		   (sizeof(Cell) == 2) ? 1 :
 		   (sizeof(Cell) == 4) ? 2 : 3) + ausize;
+  Cell sizebyte = (ausize << 5) + (charsize << 3) + (cellsize << 1) +
+#ifdef WORDS_BIGENDIAN
+       0
+#else
+       1
+#endif
+    ;
 
 #ifndef DOUBLY_INDIRECT
   check_sum = checksum(symbols);
@@ -372,34 +385,19 @@ Address loader(FILE *imagefile, char* filename)
     }
     preamblesize+=8;
   } while(memcmp(magic,"Gforth2",7));
+  magic7 = magic[7];
   if (debug) {
-    magic[8]='\0';
-    fprintf(stderr,"Magic found: %s\n", magic);
+    magic[7]='\0';
+    fprintf(stderr,"Magic found: %s ", magic);
+    print_sizes(magic7);
   }
 
-  if(magic[7] != (ausize << 5) + (charsize << 3) + (cellsize << 1) +
-#ifdef WORDS_BIGENDIAN
-       0
-#else
-       1
-#endif
-       )
-    { fprintf(stderr,"This image is %d bit cell, %d bit char, %d bit address unit %s-endian,\n"
-	      "whereas the machine is %d bit cell, %d bit char, %d bit address unit, %s-endian.\n", 
-	      (1<<((magic[7]>>1)&3))*8,
-	      (1<<((magic[7]>>3)&3))*8,
-	      (1<<((magic[7]>>5)&3))*8,
-	      endianstring[magic[7]&1],
-	      (1<<cellsize)*8,
-	      (1<<charsize)*8,
-	      (1<<ausize)*8,
-	      endianstring[
-#ifdef WORDS_BIGENDIAN
-		      0
-#else
-		      1
-#endif
-		      ]);
+  if (magic7 != sizebyte)
+    {
+      fprintf(stderr,"This image is:         ");
+      print_sizes(magic7);
+      fprintf(stderr,"whereas the machine is ");
+      print_sizes(sizebyte);
       exit(-2);
     };
 
@@ -457,6 +455,9 @@ Address loader(FILE *imagefile, char* filename)
   return imp;
 }
 
+/* index of last '/' or '\' in file, 0 if there is none. !! Hmm, could
+   be implemented with strrchr and the separator should be
+   OS-dependent */
 int onlypath(char *file)
 {
   int i;
@@ -471,13 +472,15 @@ int onlypath(char *file)
 FILE *openimage(char *fullfilename)
 {
   FILE *image_file;
+  char * expfilename = tilde_cstr(fullfilename, strlen(fullfilename), 1);
 
-  image_file=fopen(fullfilename,"rb");
+  image_file=fopen(expfilename,"rb");
   if (image_file!=NULL && debug)
-    fprintf(stderr, "Opened image file: %s\n", fullfilename);
+    fprintf(stderr, "Opened image file: %s\n", expfilename);
   return image_file;
 }
 
+/* try to open image file concat(path[0:len],imagename) */
 FILE *checkimage(char *path, int len, char *imagename)
 {
   int dirlen=len;
@@ -493,6 +496,7 @@ FILE *checkimage(char *path, int len, char *imagename)
 FILE * open_image_file(char * imagename, char * path)
 {
   FILE * image_file=NULL;
+  char *origpath=path;
   
   if(strchr(imagename, '/')==NULL) {
     /* first check the directory where the exe file is in !! 01may97jaw */
@@ -513,7 +517,7 @@ FILE * open_image_file(char * imagename, char * path)
 
   if (!image_file) {
     fprintf(stderr,"%s: cannot open image file %s in path %s for reading\n",
-	    progname, imagename, path);
+	    progname, imagename, origpath);
     exit(1);
   }
 
@@ -543,7 +547,7 @@ UCell convsize(char *s, UCell elemsize)
       m=1024*1024*1024;
     else if (strcmp(endp,"T")==0) {
 #if (SIZEOF_CHAR_P > 4)
-      m=1024*1024*1024*1024;
+      m=1024L*1024*1024*1024;
 #else
       fprintf(stderr,"%s: size specification \"%s\" too large for this machine\n", progname, endp);
       exit(1);
@@ -669,12 +673,15 @@ int main(int argc, char **argv, char **env)
 
 #ifdef INCLUDE_IMAGE
   set_stack_sizes((ImageHeader *)image);
-  relocate(image, reloc_bits, ((ImageHeader*)&image)->image_size, (Label*)engine(0, 0, 0, 0, 0));
+  if(((ImageHeader *)image)->base != image)
+    relocate(image, reloc_bits, ((ImageHeader *)image)->image_size,
+	     (Label*)engine(0, 0, 0, 0, 0));
   alloc_stacks((ImageHeader *)image);
 #else
   image_file = open_image_file(imagename, path);
   image = loader(image_file, imagename);
 #endif
+  gforth_header=(ImageHeader *)image; /* used in SIGSEGV handler */
 
   {
     char path2[strlen(path)+1];
